#define VALVE_IMPLEMENTATION
#define CSR_EXECUTABLE
#include "csRadar.h"

// gcc -Wall -fsanitize=address csRadar.c -o csRadar -lm

int main( int argc, char *argv[] )
{
	csr_api api = 
	{
		.padding = 128.f,
		.resolution = 1024,
		.write_txt = 1,
		.api_version = csr_api_version,
		.sampling_mode = k_EMSAA_RGSS
	};

	int output_set = 0;
	char *extension = NULL;

	char *arg;
	while( csr_argp( argc, argv ) )
	{
		if( (arg = csr_arg()) )
		{
			if( api.num_strings == 20 )
			{
				log_error( "Too many arguments! Max 20\n" );
				goto IL_CSR_EXIT;
			}
			
			api.strings[ api.num_strings ++ ] = arg;
		}
		
		if( (arg = csr_opt_arg( 'o' )) )
		{
			strcpy( api.output_path, arg );
			csr_path_winunix( api.output_path );
			
			output_set = 1;
		}
		
		if( (arg = csr_opt_arg( 'g' )) )
		{
			fs_set_gameinfo( arg );
		}
		
		if( (arg = csr_opt_arg( 'r' )) )
		{
			api.resolution = atoi( arg );
		}
		
		if( (arg = csr_long_opt_arg( "padding" )) )
		{
			api.padding = atof( arg );
		}
		
		if( (arg = csr_long_opt_arg( "multi-sample" )) )
		{
			if( !strcmp( arg, "none" ))
			{
				api.sampling_mode = k_EMSAA_none;
			}
			else if( !strcmp( arg, "rgss" ))
			{
				api.sampling_mode = k_EMSAA_RGSS;
			}
			else if( !strcmp( arg, "2x" ))
			{
				api.sampling_mode = k_EMSAA_2x2;
			}
			else if( !strcmp( arg, "8r" ))
			{
				api.sampling_mode = k_EMSAA_8R;
			}
			else
			{
				log_error( "Invalid sampling pattern '%s'\n", arg );
				goto IL_CSR_EXIT;
			}
		}
		
		if( (arg = csr_long_opt_arg( "extension" )) )
		{
			extension = arg;
		}

		if( csr_opt( 'v' ) || csr_long_opt( "version" ) )
		{
			printf( "csRadar build: %u, api_version: %u\n", csr_build, csr_api_version );
			goto IL_CSR_EXIT;
		}
		
		if( csr_opt( 'h' ) || csr_long_opt( "help" ) )
		{
			// Display help
			printf
			( 
				"csRadar Copyright (C) 2021 Harry Godden (hgn)\n"
				"\n"
				"Usage: ./csRadar map.vmf -g \"/gamedir/gameinfo.txt\" layout cover\n"
				"   VMF file is first, then any other arguments (eg. layout, cover), will specify\n"
				"   visgroups to be rendered into individual files\n"
				"   No visgroups specified will simply draw everything\n"
				"\n"
				"Options:\n"
				"   -g <gameinfo.txt path>         Required if you are loading models\n"
				"   -r 1024                        Output resolution\n"
				"   -o <output>                    Specify output name/path\n"
				"   --padding=128                  When cropping radar, add padding units to border\n"
				//"   --standard-layers              Use standard TAR layers/groups\n"
				"   --write-normals                Enable normals as an output stream\n"
				"   --write-origins                Enable entity origins as an output stream\n"
				"   --no-txt                       Don't create matching radar txt\n"
				"   --multi-sample=RGSS            [ none, 2x, rgss, 8r ]\n"
				"   --extension=TAR                Use an extension binary instead\n"
				"\n"
				"   -v --version                   Display program version\n"
				"   -h --help                      Display this help text\n"
			);
			
			goto IL_CSR_EXIT;
		}
	}
	
	if( api.num_strings )
	{
		// Path handling
		if( !output_set )
		{
			strcpy( api.output_path, api.strings[0] );
			csr_stripext( api.output_path );
		}
		
		char *base_name;
		if( !(base_name = csr_findext( api.output_path, '/' ) ))
		{
			base_name = api.output_path;
		}
		
		strcpy( api.vmf_name, base_name );
		
		log_info( "output_path: '%s'\n", api.output_path );
		log_info( "vmf_name: '%s'\n", api.vmf_name );

		api.map = vmf_init( api.strings[0] );
		if( api.map )
		{
			if( !extension )
				extension = "csRadarFree";
		
			csr_so ext = csr_libopen( extension );
			
			if( ext )
			{
				void (*csr_ext_main)(csr_api *);
				void (*csr_ext_exit)(csr_api *);
				
				csr_ext_main = csr_get_proc( ext, "csr_ext_main" );
				csr_ext_exit = csr_get_proc( ext, "csr_ext_exit" );
					
				if( csr_ext_main && csr_ext_exit )
				{
					csr_ext_main( &api );
					
					// Do other
					
					csr_ext_exit( &api );
				}
				else
				{
					csr_liberr();
				}
				
				csr_libclose( ext );
			}
			else
			{
				csr_liberr();
			}
			
			vmf_free( api.map );
		}
		else
		{
			log_error( "Could not load VMF\n" );
		}
	}
	else
	{
		log_error( "Missing required argument: mapfile\n" );
	}
	
IL_CSR_EXIT:
	fs_exit();
	return 0;
}
