// Copyright (C) 2021 Harry Godden (hgn)

// Logging with colour
// ======================================================================================

void log_success( const char *fmt, ... );
void log_info( const char *fmt, ... );
void log_warn( const char *fmt, ... );
void log_error( const char *fmt, ... );

#ifdef CSR_EXECUTABLE

#define KNRM  "\x1B[0m"
#define KRED  "\x1B[31m"
#define KGRN  "\x1B[32m"
#define KYEL  "\x1B[33m"
#define KBLU  "\x1B[34m"
#define KMAG  "\x1B[35m"
#define KCYN  "\x1B[36m"
#define KWHT  "\x1B[37m"

#define CSR_LOG_WRITE( FILE, PREFIX ) 	\
	fprintf( FILE, PREFIX ); 								\
	va_list args; 												\
	va_start( args, fmt ); 									\
	vfprintf( FILE, fmt, args ); 							\
	va_end( args ); 											\
	fprintf( FILE, KNRM );

void log_success( const char *fmt, ... ) { CSR_LOG_WRITE( stdout,	(KGRN "success" KWHT "| " KGRN) ); }
void log_info( const char *fmt, ... )    { CSR_LOG_WRITE( stdout,	(KNRM "   info" KWHT "| " KNRM) ); }
void log_warn( const char *fmt, ... )    { CSR_LOG_WRITE( stdout,	(KYEL "   warn" KWHT "| " KYEL) ); }
void log_error( const char *fmt, ... )   { CSR_LOG_WRITE( stderr,	(KRED "  error" KWHT "| " KRED) ); }

// TODO: Remove this?
const char *csr_prog_msg;
void csr_prog_begin( const char *msg )
{
	csr_prog_msg = msg;
}

void csr_prog_update( float const percent )
{
	int const k_track_length = 40;

	printf( "\r %s %d%% |", csr_prog_msg, (int)(percent*100.f) );
	
	// Calculate how many steps travelled
	int cur_pos = ((float)k_track_length * (1.f-percent));
	
	for( int i = 0; i < cur_pos; i ++ )
		printf( " " );
		
	printf( "🚂🚃🚃🚃🚃" );
	
	for( int i = 0; i < k_track_length-cur_pos; i ++ )
		printf( " " );
		
	printf( "|" );
	
	fflush( stdout );
}

void csr_prog_end(void)
{
	csr_prog_update( 1.f );
	printf( "\n" );
}

#endif
